@@ -27,7 +27,7 @@
 #ifdef HAVE_USRP0
 #include "fpga_rev0.h"
 #endif
-#ifdef HAVE_USRP1
+#if defined(HAVE_USRP1) || defined(HAVE_USRP2)
 #include "fpga_rev1.h"
 #endif
 
