@@ -20,10 +20,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "time_stuff.h"
 
 #include <sys/time.h>
+#ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
+#endif
 #include <unistd.h>
 
 static double
@@ -35,12 +41,16 @@ timeval_to_secs (struct timeval *tv)
 double
 get_cpu_usage (void)
 {
+#ifdef HAVE_GETRUSAGE
   struct rusage	ru;
 
   if (getrusage (RUSAGE_SELF, &ru) != 0)
     return 0;
 
   return timeval_to_secs (&ru.ru_utime) + timeval_to_secs (&ru.ru_stime);
+#else
+  return 0;	/* FIXME */
+#endif
 }
 
 /*
