@@ -49,7 +49,8 @@ class  fusb_ephandle;
 /*!
  * \brief base class for usrp operations
  */
-class usrp_basic {
+class usrp_basic
+{
 private:
   // NOT IMPLEMENTED
   usrp_basic (const usrp_basic &rhs);			// no copy constructor
@@ -148,7 +149,6 @@ public:
    */
   std::string read_eeprom (int i2c_addr, int eeprom_offset, int len);
 
-
   // ----------------------------------------------------------------
   // Low level implementation routines.
   // You probably shouldn't be using these...
@@ -195,7 +195,7 @@ public:
    * \param value	8-bit value
    * \returns true iff successful
    */
-  bool _read_9862 (int which_codec, int regno, unsigned char *value);
+  bool _read_9862 (int which_codec, int regno, unsigned char *value) const;
 
   /*!
    * \brief Read AD9862 register.
@@ -203,13 +203,14 @@ public:
    * \param regno	6-bit register number
    * \returns register value if successful, else READ_FAILED
    */
-  int  _read_9862 (int which_codec, int regno);
+  int  _read_9862 (int which_codec, int regno) const;
 };
 
-/*!
+/*!
  * \brief class for accessing the receive side of the USRP
  */
-class usrp_basic_rx : public usrp_basic {
+class usrp_basic_rx : public usrp_basic 
+{
 private:
   fusb_devhandle	*d_devhandle;
   fusb_ephandle		*d_ephandle;
@@ -217,8 +218,10 @@ private:
   bool			 d_first_read;
   bool			 d_rx_enable;
 
- protected:
-  usrp_basic_rx (int which_board);	// throws if trouble
+protected:
+  int			 d_dbid[2];		// Rx daughterboard ID's
+
+  usrp_basic_rx (int which_board);		// throws if trouble
 
   // called after construction in base class to derived class order
   bool initialize ();
@@ -226,6 +229,12 @@ private:
   bool set_rx_enable (bool on);
   bool rx_enable () const { return d_rx_enable; }
 
+  bool disable_rx ();		// conditional disable, return prev state
+  void restore_rx (bool on);	// conditional set
+
+  void probe_rx_slots (bool verbose);
+  int  dboard_to_slot (int dboard) { return (dboard << 1) | 1; }
+
 public:
   ~usrp_basic_rx ();
 
@@ -263,12 +272,103 @@ public:
   //! sampling rate of A/D converter
   virtual long adc_freq () const { return fpga_master_clock_freq () / 2; } // 64M
 
+  /*!
+   * \brief Return daughterboard ID for given Rx daughterboard slot [0,1].
+   *
+   * \param which_board		[0,1] which Rx daughterboard
+   *
+   * \return daughterboard id >= 0 if successful
+   * \return -1 if no daugherboard
+   * \return -2 if invalid EEPROM on daughterboard
+   */
+  int daughterboard_id (int which_dboard) const { return d_dbid[which_dboard & 0x1]; }
+
+  // ----------------------------------------------------------------
+  // routines for controlling the Programmable Gain Amplifier
+  /*!
+   * \brief Set Programmable Gain Amplifier (PGA)
+   *
+   * \param which	which A/D [0,3]
+   * \param gain_in_db	gain value (linear in dB)
+   *
+   * gain is rounded to closest setting supported by hardware.
+   *
+   * \returns true iff sucessful.
+   *
+   * \sa pga_min(), pga_max(), pga_db_per_step()
+   */
+  bool set_pga (int which, double gain_in_db);
+
+  /*!
+   * \brief Return programmable gain amplifier gain setting in dB.
+   *
+   * \param which	which A/D [0,3]
+   */
+  double pga (int which) const;
+
+  /*!
+   * \brief Return minimum legal PGA gain in dB.
+   */
+  double pga_min () const { return 0.0; }
+
+  /*!
+   * \brief Return maximum legal PGA gain in dB.
+   */
+  double pga_max () const { return 20.0; }
+
+  /*!
+   * \brief Return hardware step size of PGA (linear in dB).
+   */
+  double pga_db_per_step () const { return 1.0; }
+
+  /*!
+   * \brief Write direction register (output enables) for pins that go to daughterboard.
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param value		value to write into register
+   * \param mask		which bits of value to write into reg
+   *
+   * Each d'board has 16-bits of general purpose i/o.
+   * Setting the bit makes it an output from the FPGA to the d'board.
+   *
+   * This register is initialized based on a value stored in the
+   * d'board EEPROM.  In general, you shouldn't be using this routine
+   * without a very good reason.  Using this method incorrectly will
+   * kill your USRP motherboard and/or daughterboard.
+   */
+  bool _write_oe (int which_dboard, int value, int mask);
+
+  /*!
+   * \brief Write daughterboard i/o pin value
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param value		value to write into register
+   * \param mask		which bits of value to write into reg
+   */
+  bool write_io (int which_dboard, int value, int mask);
+
+  /*!
+   * \brief Read daughterboard i/o pin value
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param value		output
+   */
+  bool read_io (int which_dboard, int *value);
+
+  /*!
+   * \brief Read daughterboard i/o pin value
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \returns register value if successful, else READ_FAILED
+   */
+  int read_io (int which_dboard);
 };
 
-/*!
+/*!
  * \brief class for accessing the transmit side of the USRP
  */
-class usrp_basic_tx : public usrp_basic {
+class usrp_basic_tx : public usrp_basic 
+{
 private:
   fusb_devhandle	*d_devhandle;
   fusb_ephandle		*d_ephandle;
@@ -277,7 +377,9 @@ private:
   bool			 d_tx_enable;
 
  protected:
-  usrp_basic_tx (int which_board);	// throws if trouble
+  int			 d_dbid[2];		// Tx daughterboard ID's
+
+  usrp_basic_tx (int which_board);		// throws if trouble
 
   // called after construction in base class to derived class order
   bool initialize ();
@@ -285,6 +387,12 @@ private:
   bool set_tx_enable (bool on);
   bool tx_enable () const { return d_tx_enable; }
 
+  bool disable_tx ();		// conditional disable, return prev state
+  void restore_tx (bool on);	// conditional set
+
+  void probe_tx_slots (bool verbose);
+  int  dboard_to_slot (int dboard) { return (dboard << 1) | 0; }
+
 public:
 
   ~usrp_basic_tx ();
@@ -330,6 +438,95 @@ public:
   //! sampling rate of D/A converter
   virtual long dac_freq () const { return fpga_master_clock_freq (); } // 128M
 
-};
+  /*!
+   * \brief Return daughterboard ID for given Tx daughterboard slot [0,1].
+   *
+   * \return daughterboard id >= 0 if successful
+   * \return -1 if no daugherboard
+   * \return -2 if invalid EEPROM on daughterboard
+   */
+  int daughterboard_id (int which_dboard) const { return d_dbid[which_dboard & 0x1]; }
 
+  // ----------------------------------------------------------------
+  // routines for controlling the Programmable Gain Amplifier
+  /*!
+   * \brief Set Programmable Gain Amplifier (PGA)
+   *
+   * \param which	which D/A [0,3]
+   * \param gain_in_db	gain value (linear in dB)
+   *
+   * gain is rounded to closest setting supported by hardware.
+   * Note that DAC 0 and DAC 1 share a gain setting as do DAC 2 and DAC 3.
+   * Setting DAC 0 affects DAC 1 and vice versa.  Same with DAC 2 and DAC 3.
+   *
+   * \returns true iff sucessful.
+   *
+   * \sa pga_min(), pga_max(), pga_db_per_step()
+   */
+  bool set_pga (int which, double gain_in_db);
+
+  /*!
+   * \brief Return programmable gain amplifier gain in dB.
+   *
+   * \param which	which D/A [0,3]
+   */
+  double pga (int which) const;
+
+  /*!
+   * \brief Return minimum legal PGA gain in dB.
+   */
+  double pga_min () const { return -20.0; }
+
+  /*!
+   * \brief Return maximum legal PGA gain in dB.
+   */
+  double pga_max () const { return 0.0; }
+
+  /*!
+   * \brief Return hardware step size of PGA (linear in dB).
+   */
+  double pga_db_per_step () const { return 0.1; }
+
+  /*!
+   * \brief Write direction register (output enables) for pins that go to daughterboard.
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param value		value to write into register
+   * \param mask		which bits of value to write into reg
+   *
+   * Each d'board has 16-bits of general purpose i/o.
+   * Setting the bit makes it an output from the FPGA to the d'board.
+   *
+   * This register is initialized based on a value stored in the
+   * d'board EEPROM.  In general, you shouldn't be using this routine
+   * without a very good reason.  Using this method incorrectly will
+   * kill your USRP motherboard and/or daughterboard.
+   */
+  bool _write_oe (int which_dboard, int value, int mask);
+
+  /*!
+   * \brief Write daughterboard i/o pin value
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param value		value to write into register
+   * \param mask		which bits of value to write into reg
+   */
+  bool write_io (int which_dboard, int value, int mask);
+
+  /*!
+   * \brief Read daughterboard i/o pin value
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param value		return value
+   */
+  bool read_io (int which_dboard, int *value);
+
+  /*!
+   * \brief Read daughterboard i/o pin value
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \returns register value if successful, else READ_FAILED
+   */
+  int read_io (int which_dboard);
+};
 #endif
