@@ -27,10 +27,11 @@
 #ifndef INCLUDED_USRP_DAUGHTERBOARDS_H
 #define INCLUDED_USRP_DAUGHTERBOARDS_H
 
-#define	USRP_DBID_TX_DEBUG	0x0000
-#define	USRP_DBID_RX_DEBUG	0x0001
-#define	USRP_DBID_4702_SINGLE	0x0002
-#define	USRP_DBID_4702_DOUBLE	0x0003
+#define	USRP_DBID_BASIC_TX		0x0000
+#define	USRP_DBID_BASIC_RX		0x0001
+#define	USRP_DBID_EXPERIMENTAL_TX	0xfffe	// homebrew, etc
+#define	USRP_DBID_EXPERIMENTAL_RX	0xffff
+
 
 
 #endif /* INCLUDED_USRP_DAUGHTERBOARDS_H */
