@@ -35,6 +35,7 @@
 #define _USRP_PRIMS_H_
 
 #include <usrp_slots.h>
+#include <string>
 
 static const int USRP_HASH_SIZE = 16;
 
@@ -69,6 +70,7 @@ struct usb_device *usrp_find_device (int nth, bool fx2_ok_p = false);
 bool usrp_usrp_p (struct usb_device *q);		//< is this a USRP
 bool usrp_usrp0_p (struct usb_device *q);		//< is this a USRP Rev 0
 bool usrp_usrp1_p (struct usb_device *q);		//< is this a USRP Rev 1
+bool usrp_usrp2_p (struct usb_device *q);		//< is this a USRP Rev 2
 bool usrp_fx2_p (struct usb_device *q);			//< is this an unconfigured Cypress FX2
 
 bool usrp_unconfigured_usrp_p (struct usb_device *q);	//< some kind of unconfigured USRP
@@ -250,4 +252,34 @@ bool usrp_read_aux_adc (struct usb_dev_handle *udh, int slot,
 			int which_adc, int *value);
 
 
+/*!
+ * \brief usrp daughterboard id to name mapping
+ *
+ * Returns 0 length string if dbid not known.
+ */
+const std::string usrp_dbid_to_string (unsigned short dbid);
+
+
+enum usrp_dbeeprom_status_t { UDBE_OK, UDBE_BAD_SLOT, UDBE_NO_EEPROM, UDBE_INVALID_EEPROM };
+
+struct usrp_dboard_eeprom {
+  unsigned short	id;		// d'board identifier code
+  unsigned short	oe;		// fpga output enables:
+					//   If bit set, i/o pin is an output from FPGA.
+  short			offset[2];	// ADC/DAC offset correction
+};
+
+/*!
+ * \brief Read and return parsed daughterboard eeprom
+ */
+usrp_dbeeprom_status_t
+usrp_read_dboard_eeprom (struct usb_dev_handle *udh,
+			 int slot_id, usrp_dboard_eeprom *eeprom);
+
+/*!
+ * \brief write ADC/DAC offset calibration constants to d'board eeprom
+ */
+bool usrp_write_dboard_offsets (struct usb_dev_handle *udh, int slot_id,
+				short offset0, short offset1);
+
 #endif /* _USRP_PRIMS_H_ */
