@@ -23,7 +23,8 @@
 #include <usrp_standard.h>
 
 #include "usrp_prims.h"
-#include "fpga_regs.h"
+#include "fpga_regs_common.h"
+#include "fpga_regs_standard.h"
 #include <stdexcept>
 #include <assert.h>
 #include <math.h>
@@ -86,16 +87,41 @@ compute_freq_control_word_9862 (double master_freq, double target_freq,
 
 // ----------------------------------------------------------------
 
-usrp_standard_rx::usrp_standard_rx (int which_board, unsigned int decim_rate,
-				    int mode)
-  : usrp_basic_rx (which_board)
+static int 
+real_rx_mux_value (int mux, int nchan)
 {
-  set_decim_rate (decim_rate);
+  if (mux != -1)
+    return mux;
+
+  return 0x32103210;
+}
+
+usrp_standard_rx::usrp_standard_rx (int which_board,
+				    unsigned int decim_rate,
+				    int nchan, int mux, int mode)
+  : usrp_basic_rx (which_board),
+    d_nchan (1), d_sw_mux (0x0), d_hw_mux (0x0)
+{
+  if (!set_decim_rate (decim_rate)){
+    fprintf (stderr, "usrp_standard_rx: set_decim_rate failed\n");
+    throw std::runtime_error ("usrp_standard_rx::ctor");
+  }
+  if (!set_nchannels (nchan)){
+    fprintf (stderr, "usrp_standard_rx: set_nchannels failed\n");
+    throw std::runtime_error ("usrp_standard_rx::ctor");
+  }
+  if (!set_mux (real_rx_mux_value (mux, nchan))){
+    fprintf (stderr, "usrp_standard_rx: set_mux failed\n");
+    throw std::runtime_error ("usrp_standard_rx::ctor");
+  }
+  if (!set_fpga_mode (mode)){
+    fprintf (stderr, "usrp_standard_rx: set_fpga_mode failed\n");
+    throw std::runtime_error ("usrp_standard_rx::ctor");
+  }
 
   for (int i = 0; i < MAX_CHAN; i++)
     set_rx_freq (i, 0);
 
-  set_fpga_mode (mode);
 }
 
 usrp_standard_rx::~usrp_standard_rx ()
@@ -115,12 +141,15 @@ usrp_standard_rx::initialize ()
 }
 
 usrp_standard_rx *
-usrp_standard_rx::make (int which_board, unsigned int decim_rate, int mode)
+usrp_standard_rx::make (int which_board,
+			unsigned int decim_rate,
+			int nchan, int mux, int mode)
 {
   usrp_standard_rx *u = 0;
   
   try {
-    u = new usrp_standard_rx (which_board, decim_rate, mode);
+    u = new usrp_standard_rx (which_board, decim_rate,
+			      nchan, mux, mode);
     if (!u->initialize ()){
       fprintf (stderr, "usrp_standard_rx::make failed to initialize\n");
       throw std::runtime_error ("usrp_standard_rx::make");
@@ -139,21 +168,121 @@ bool
 usrp_standard_rx::set_decim_rate  (unsigned int rate)
 {
   d_decim_rate = rate;
-  set_usb_data_rate ((adc_freq () / rate) * (2 * sizeof (short)));
-
-  bool ok;
-
-  if (rx_enable ()){
-    set_rx_enable (false);
-    ok = _write_fpga_reg (FR_DECIM_RATE, d_decim_rate - 1);
-    set_rx_enable (true);
-  }
-  else
-    ok = _write_fpga_reg (FR_DECIM_RATE, d_decim_rate - 1);
+  set_usb_data_rate ((adc_freq () / rate * nchannels ())
+		     * (2 * sizeof (short)));
 
+  bool s = disable_rx ();
+  bool ok = _write_fpga_reg (FR_DECIM_RATE, d_decim_rate - 1);
+  restore_rx (s);
   return ok;
 }
 
+bool usrp_standard_rx::set_nchannels (int nchan)
+{
+  if (!(nchan == 1 || nchan == 2 || nchan == 4))
+    return false;
+
+  d_nchan = nchan;
+
+  return write_hw_mux_reg ();
+}
+
+
+// map software mux value to hw mux value
+//
+// Software mux value:
+//
+//    3                   2                   1                       
+//  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+// +-------+-------+-------+-------+-------+-------+-------+-------+
+// |   Q3  |   I3  |   Q2  |   I2  |   Q1  |   I1  |   Q0  |   I0  |
+// +-------+-------+-------+-------+-------+-------+-------+-------+
+//
+// Each 4-bit I field is either 0,1,2,3
+// Each 4-bit Q field is either 0,1,2,3 or 0xf (input is const zero)
+// All Q's must be 0xf or none of them may be 0xf
+//
+//
+// Hardware mux value:
+//
+//    3                   2                   1                       
+//  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+// +-----------------------+-------+-------+-------+-------+-+-----+
+// |      must be zero     | Q3| I3| Q2| I2| Q1| I1| Q0| I0|Z| NCH |
+// +-----------------------+-------+-------+-------+-------+-+-----+
+
+
+static bool
+map_sw_mux_to_hw_mux (int sw_mux, int *hw_mux_ptr)
+{
+  // confirm that all I's are either 0,1,2,3 
+
+  for (int i = 0; i < 8; i += 2){
+    int t = (sw_mux >> (4 * i)) & 0xf;
+    if (!(0 <= t && t <= 3))
+      return false;
+  }
+
+  // confirm that all Q's are either 0,1,2,3 or 0xf
+
+  for (int i = 1; i < 8; i += 2){
+    int t = (sw_mux >> (4 * i)) & 0xf;
+    if (!(t == 0xf || (0 <= t && t <= 3)))
+      return false;
+  }
+
+  // confirm that all Q inputs are 0xf (const zero input),
+  // or none of them are 0xf
+
+  int q_and = 1;
+  int q_or =  0;
+
+  for (int i = 0; i < 4; i++){
+    int qx_is_0xf = ((sw_mux >> (8 * i + 4)) & 0xf) == 0xf;
+    q_and &= qx_is_0xf;
+    q_or  |= qx_is_0xf;
+  }
+
+  if (q_and || !q_or){		// OK
+    int hw_mux_value = 0;
+
+    for (int i = 0; i < 8; i++){
+      int t = (sw_mux >> (4 * i)) & 0x3;
+      hw_mux_value |= t << (2 * i + 4);
+    }
+
+    if (q_and)
+      hw_mux_value |= 0x8;	// all Q's zero
+
+    *hw_mux_ptr = hw_mux_value;
+    return true;
+  }
+  else
+    return false;
+}
+
+bool
+usrp_standard_rx::set_mux (int mux)
+{
+  if (!map_sw_mux_to_hw_mux (mux, &d_hw_mux))
+    return false;
+
+  // fprintf (stderr, "sw_mux = 0x%08x  hw_mux = 0x%08x\n", mux, d_hw_mux);
+
+  d_sw_mux = mux;
+  return write_hw_mux_reg ();
+}
+
+bool
+usrp_standard_rx::write_hw_mux_reg ()
+{
+  bool s = disable_rx ();
+  bool ok = _write_fpga_reg (FR_RX_MUX, d_hw_mux | d_nchan);
+  restore_rx (s);
+  return ok;
+}
+
+
 bool
 usrp_standard_rx::set_rx_freq (int channel, double freq)
 {
@@ -171,6 +300,11 @@ usrp_standard_rx::set_rx_freq (int channel, double freq)
 unsigned int
 usrp_standard_rx::decim_rate () const { return d_decim_rate; }
 
+int
+usrp_standard_rx::nchannels () const { return d_nchan; }
+
+int
+usrp_standard_rx::mux () const { return d_sw_mux; }
 
 double 
 usrp_standard_rx::rx_freq (int channel) const
@@ -206,15 +340,47 @@ static unsigned char tx_regs_use_nco[] = {
 };
 
 
-usrp_standard_tx::usrp_standard_tx (int which_board, unsigned int interp_rate)
-  : usrp_basic_tx (which_board)
+static int
+real_tx_mux_value (int mux, int nchan)
+{
+  if (mux != -1)
+    return mux;
+
+  switch (nchan){
+  case 1:
+    return 0x0098;
+  case 2:
+    return 0xba98;
+  default:
+    assert (0);
+  }
+}
+
+usrp_standard_tx::usrp_standard_tx (int which_board,
+				    unsigned int interp_rate,
+				    int nchan, int mux)
+  : usrp_basic_tx (which_board),
+    d_sw_mux (0x8), d_hw_mux (0x81)
 {
   if (!usrp_9862_write_many_all (d_udh, tx_regs_use_nco, sizeof (tx_regs_use_nco))){
     fprintf (stderr, "usrp_standard_tx: failed to init AD9862 TX regs\n");
-    throw std::runtime_error ("usrp_standard_tx/init_9862");
+    throw std::runtime_error ("usrp_standard_tx::ctor");
   }
 
-  set_interp_rate (interp_rate);
+  if (!set_interp_rate (interp_rate)){
+    fprintf (stderr, "usrp_standard_tx: set_interp_rate failed\n");
+    throw std::runtime_error ("usrp_standard_tx::ctor");
+  }
+
+  if (!set_nchannels (nchan)){
+    fprintf (stderr, "usrp_standard_tx: set_nchannels failed\n");
+    throw std::runtime_error ("usrp_standard_tx::ctor");
+  }
+
+  if (!set_mux (real_tx_mux_value (mux, nchan))){
+    fprintf (stderr, "usrp_standard_tx: set_mux failed\n");
+    throw std::runtime_error ("usrp_standard_tx::ctor");
+  }
 
   for (int i = 0; i < MAX_CHAN; i++){
     d_tx_modulator_shadow[i] = (TX_MODULATOR_DISABLE_NCO
@@ -241,12 +407,14 @@ usrp_standard_tx::initialize ()
 }
 
 usrp_standard_tx *
-usrp_standard_tx::make (int which_board, unsigned int interp_rate)
+usrp_standard_tx::make (int which_board,
+			unsigned int interp_rate,
+			int nchan, int mux)
 {
   usrp_standard_tx *u = 0;
   
   try {
-    u = new usrp_standard_tx (which_board, interp_rate);
+    u = new usrp_standard_tx (which_board, interp_rate, nchan, mux);
     if (!u->initialize ()){
       fprintf (stderr, "usrp_standard_tx::make failed to initialize\n");
       throw std::runtime_error ("usrp_standard_tx::make");
@@ -272,22 +440,43 @@ usrp_standard_tx::set_interp_rate (unsigned int rate)
   }
 
   d_interp_rate = rate;
-  set_usb_data_rate ((dac_freq () / rate) * (2 * sizeof (short)));
+  set_usb_data_rate ((dac_freq () / rate * nchannels ())
+		     * (2 * sizeof (short)));
 
   // We're using the interp by 4 feature of the 9862 so that we can
   // use its fine modulator.  Thus, we reduce the FPGA's interpolation rate
   // by a factor of 4.
 
-  bool ok;
+  bool s = disable_tx ();
+  bool ok = _write_fpga_reg (FR_INTERP_RATE, d_interp_rate/4 - 1);
+  restore_tx (s);
+  return ok;
+}
 
-  if (tx_enable ()){
-    set_tx_enable (false);
-    ok = _write_fpga_reg (FR_INTERP_RATE, d_interp_rate/4 - 1);
-    set_tx_enable (true);
-  }
-  else
-    ok = _write_fpga_reg (FR_INTERP_RATE, d_interp_rate/4 - 1);    
+bool
+usrp_standard_tx::set_nchannels (int nchan)
+{
+  if (!(nchan == 1 || nchan == 2))
+    return false;
 
+  d_nchan = nchan;
+  return write_hw_mux_reg ();
+}
+
+bool
+usrp_standard_tx::set_mux (int mux)
+{
+  d_sw_mux = mux;
+  d_hw_mux = mux << 4;
+  return write_hw_mux_reg ();
+}
+
+bool
+usrp_standard_tx::write_hw_mux_reg ()
+{
+  bool s = disable_tx ();
+  bool ok = _write_fpga_reg (FR_TX_MUX, d_hw_mux | d_nchan);
+  restore_tx (s);
   return ok;
 }
 
@@ -438,6 +627,12 @@ usrp_standard_tx::set_coarse_modulator (int channel, coarse_mod_t cm)
 unsigned int
 usrp_standard_tx::interp_rate () const { return d_interp_rate; }
 
+int
+usrp_standard_tx::nchannels () const { return d_nchan; }
+
+int
+usrp_standard_tx::mux () const { return d_sw_mux; }
+
 double
 usrp_standard_tx::tx_freq (int channel) const
 {
