@@ -35,16 +35,23 @@ class usrp_standard_rx : public usrp_basic_rx
  private:
   static const int	MAX_CHAN = 4;
   unsigned int		d_decim_rate;
+  int			d_nchan;
+  int			d_sw_mux;
+  int			d_hw_mux;
   double		d_rx_freq[MAX_CHAN];
 
-
  protected:
-  usrp_standard_rx (int which_board, unsigned int decim_rate,
-		    int mode = 0);	// throws if trouble
+  usrp_standard_rx (int which_board,
+		    unsigned int decim_rate,
+		    int nchan = 1,
+		    int mux = -1,
+		    int mode = 0
+		    );  // throws if trouble
 
   // called after construction in base class to derived class order
   bool initialize ();
 
+  bool write_hw_mux_reg ();
 
  public:
 
@@ -59,16 +66,49 @@ class usrp_standard_rx : public usrp_basic_rx
   /*!
    * \brief invokes constructor, returns instance or 0 if trouble
    */
-  static usrp_standard_rx *make (int which_board, unsigned int decim_rate,
-				 int mode = 0);
+  static usrp_standard_rx *make (int which_board,
+				 unsigned int decim_rate,
+				 int nchan = 1,
+				 int mux = -1,
+				 int mode = 0
+				 );
   /*!
    * \brief Set decimator rate.  \p rate must be in [1, 256]
    *
    * The final complex sample rate across the USB is
-   *   adc_freq () / decim_rate ()
+   *   adc_freq () / decim_rate () * nchannels ()
    */
   bool set_decim_rate  (unsigned int rate);
 
+  /*!
+   * \brief Set number of active channels.  \p nchannels must be 1, 2 or 4.
+   *
+   * The final complex sample rate across the USB is
+   *   adc_freq () / decim_rate () * nchannels ()
+   */
+  bool set_nchannels (int nchannels);
+
+  /*!
+   * \brief Set input mux configuration.
+   *
+   * This determines which ADC (or constant zero) is connected to 
+   * each DDC input.  There are 4 DDCs.  Each has two inputs.
+   *
+   * Mux value:
+   *
+   *    3                   2                   1                       
+   *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+   * +-------+-------+-------+-------+-------+-------+-------+-------+
+   * |   Q3  |   I3  |   Q2  |   I2  |   Q1  |   I1  |   Q0  |   I0  |
+   * +-------+-------+-------+-------+-------+-------+-------+-------+
+   *
+   * Each 4-bit I field is either 0,1,2,3
+   * Each 4-bit Q field is either 0,1,2,3 or 0xf (input is const zero)
+   * All Q's must be 0xf or none of them may be 0xf
+   *
+   */
+  bool set_mux  (int mux);
+
   /*!
    * \brief set the frequency of the digital down converter.
    *
@@ -88,6 +128,8 @@ class usrp_standard_rx : public usrp_basic_rx
   // ACCESSORS
   unsigned int decim_rate () const;
   double rx_freq (int channel) const;
+  int nchannels () const;
+  int mux () const;
 };
 
 // ----------------------------------------------------------------
@@ -111,6 +153,9 @@ class usrp_standard_tx : public usrp_basic_tx
  protected:
   static const int	MAX_CHAN = 2;
   unsigned int		d_interp_rate;
+  int			d_nchan;
+  int			d_sw_mux;
+  int			d_hw_mux;
   double		d_tx_freq[MAX_CHAN];
   coarse_mod_t		d_coarse_mod[MAX_CHAN];
   unsigned char		d_tx_modulator_shadow[MAX_CHAN];
@@ -119,27 +164,78 @@ class usrp_standard_tx : public usrp_basic_tx
   usrp_standard_tx::coarse_mod_t coarse_modulator (int channel) const;
 
  protected:
-  usrp_standard_tx (int which_board, unsigned int interp_rate);	// throws if trouble
+  usrp_standard_tx (int which_board,
+		    unsigned int interp_rate,
+		    int nchan = 1,
+		    int	mux = -1
+		    );	// throws if trouble
 
   // called after construction in base class to derived class order
   bool initialize ();
 
+  bool write_hw_mux_reg ();
+
  public:
   ~usrp_standard_tx ();
 
   /*!
    * \brief invokes constructor, returns instance or 0 if trouble
    */
-  static usrp_standard_tx *make (int which_board, unsigned int interp_rate);
+  static usrp_standard_tx *make (int which_board,
+				 unsigned int interp_rate,
+				 int nchan = 1,
+				 int mux = -1
+				 );
 
   /*!
    * \brief Set interpolator rate.  \p rate must be in [4, 1024] and a multiple of 4.
    *
    * The final complex sample rate across the USB is
-   *   dac_freq () / interp_rate ()
+   *   dac_freq () / interp_rate () * nchannels ()
    */
   virtual bool set_interp_rate (unsigned int rate);
 
+  /*!
+   * \brief Set number of active channels.  \p nchannels must be 1 or 2.
+   *
+   * The final complex sample rate across the USB is
+   *   dac_freq () / decim_rate () * nchannels ()
+   */
+  bool set_nchannels  (int nchannels);
+
+  /*!
+   * \brief Set output mux configuration.
+   *
+   *     3                   2                   1                       
+   *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+   *  +-------------------------------+-------+-------+-------+-------+
+   *  |                               | DAC3  | DAC2  | DAC3  |  DAC0 |
+   *  +-------------------------------+-------+-------+-------+-------+
+   * 
+   *  There are two interpolators with complex inputs and outputs.
+   *  There are four DACs.
+   * 
+   *  Each 4-bit DACx field specifies the source for the DAC and
+   *  whether or not that DAC is enabled.  Each subfield is coded
+   *  like this: 
+   * 
+   *     3 2 1 0
+   *    +-+-----+
+   *    |E|  N  |
+   *    +-+-----+
+   * 
+   *  Where E is set if the DAC is enabled, and N specifies which
+   *  interpolator output is connected to this DAC.
+   * 
+   *   N   which interp output
+   *  ---  -------------------
+   *   0   chan 0 I
+   *   1   chan 0 Q
+   *   2   chan 1 I
+   *   3   chan 1 Q
+   */
+  bool set_mux  (int mux);
+
   /*!
    * \brief set the frequency of the digital up converter.
    *
@@ -153,7 +249,8 @@ class usrp_standard_tx : public usrp_basic_tx
   // ACCESSORS
   unsigned int interp_rate () const;
   double tx_freq (int channel) const;
-
+  int nchannels () const;
+  int mux () const;
 };
 
 #endif /* INCLUDED_USRP_STANDARD_H */
